import os

os.environ['twneo_config_path'] = r'/Users/pankaj/dev/git/twneo/twneo/config/config_dev.yaml'


from twneo.twitter.tweet_collector import TweetCollector
from twneo.neo4j.graph_creator import TweetsGraphCreator


def main():
    query = '(#byjus)'
    collector = TweetCollector()
    collector.max_result_per_query = 50
    results = collector.collect_tweets_for_query(query, max_count=50)
    graph_creator = TweetsGraphCreator()
    count = 0
    for json_resp in results:
        meta_data = json_resp['meta']
        count = count + meta_data['result_count']
        graph_creator.create_tweets_graph(json_resp)
        print(f"created for {count} records ")

if __name__ == '__main__':
    main()


import requests
import os
import json
from twneo.config.config import TWNEO_CONFIG

TWNEO_CONFIG = TWNEO_CONFIG['TWNEO_CONFIG']
bearer_token = TWNEO_CONFIG['twitter_bearer_tocken']

class TweetCollector:
    search_url = "https://api.twitter.com/2/tweets/search/recent"
    max_result_per_query = '10'

    def get_params(self, query):
        return {'query': query,
                "tweet.fields": ["public_metrics,entities,referenced_tweets"],
                "user.fields": ["location"], "expansions": "author_id",
                'max_results': self.max_result_per_query, }

    def bearer_oauth(self, r):
        """
        Method required by bearer token authentication.
        """
        r.headers["Authorization"] = f"Bearer {bearer_token}"
        r.headers["User-Agent"] = "v2FullArchiveSearchPython"
        return r

    def connect_to_endpoint(self, url, params):

        response = requests.request("GET", url, auth=self.bearer_oauth, params=params)
        print(response.status_code)
        if response.status_code != 200:
            raise Exception(response.status_code, response.text)
        return response.json()

    def collect_tweets_for_query(self, query, max_count=100):
        result = []

        params = self.get_params(query)
        # self.search_url = self.search_url + "?tweet.fields=public_metrics,entities"
        json_resp = self.connect_to_endpoint(self.search_url, params=params)
        result.append(json_resp)
        meta_data = json_resp['meta']
        count = meta_data['result_count']

        while 'next_token' in meta_data:
            if count >= max_count:
                break
            params['next_token'] = meta_data['next_token']
            json_resp = self.connect_to_endpoint(self.search_url, params=params)
            result.append(json_resp)
            count = count + meta_data['result_count']

        return result

    def collect_tweets_details(self, tweet_ids):
        params = {'ids': ','.join(tweet_ids),
                  "tweet.fields": ["public_metrics,entities,referenced_tweets"],
                  "user.fields": ["location,username"], "expansions": "author_id",
                  }
        search_url = 'https://api.twitter.com/2/tweets'
        result = self.connect_to_endpoint(search_url, params)
        return result


from neo4j import GraphDatabase
from nltk.tokenize import TweetTokenizer
from twneo.config.config import TWNEO_CONFIG

TWNEO_CONFIG = TWNEO_CONFIG['TWNEO_CONFIG']

from twneo.twitter.tweet_collector import TweetCollector

class TweetsGraphCreator:


    def __init__(self):
        self.db_name = TWNEO_CONFIG['NEO4J']['database']
        self.uri = TWNEO_CONFIG['NEO4J']['uri']
        self.user = TWNEO_CONFIG['NEO4J']['user']
        self.password = TWNEO_CONFIG['NEO4J']['password']
        self.tweet_tokenizer = TweetTokenizer()
        self.driver = GraphDatabase.driver(self.uri, auth=(self.user, self.password))

    def create_user(self, tx, user, userId , location = 'NA'):
        sql = """ MERGE (u:User {name: $user, userId:$userId})
        MERGE (l:Location {location : $location})
        MERGE (u)-[:lives_in]->(l)
         """
        tx.run(sql, user=user, location = location, userId =userId, relation='lives_in')
        print(sql, user, userId, location)

    def create_tweet(self, tx, tweetId, text=None, retweet_count=0):
        sql = """ MERGE (t:Tweet {tweetId: $tweetId})
            """
        tx.run(sql, tweetId=tweetId)
        if text is not None:
            sql = """ match (t:Tweet {tweetId: $tweetId})
            set t.text = $text
                        """
            tx.run(sql, tweetId=tweetId,text=text)
            sql = """ match (t:Tweet {tweetId: $tweetId})
                       set t.retweetCount = $retweet_count
                                   """
            tx.run(sql, tweetId=tweetId, text=text,retweet_count=retweet_count)

        print(sql, tweetId, text)


    def create_user_mention_relation(self, tx, userId,userName, tweetId, relation_name='retweets'):
        sql = "merge (t:Tweet {tweetId: $tweetId}) " \
              "merge (u:User {name: $userName, userid:$userId}) "\
              f"MERGE (u)-[:{relation_name}]->(t)"
        tx.run(sql,
               userId=userId, tweetId=tweetId, userName=userName
               )
        print(sql, userId, userName)

    def create_user_retweet_relation(self, tx, userId, tweetId, relation_name='retweets'):
        sql = "merge (t:Tweet {tweetId: $tweetId}) " \
              "merge (u:User {  userId:$userId}) " \
              f"MERGE (u)-[:{relation_name}]->(t)"
        tx.run(sql,
               userId=userId, tweetId=tweetId,
               )
        print(sql, userId)

    def create_tweets_graph(self, json_resp):
        with self.driver.session(database= self.db_name) as session:

            all_referenced_tweet_ids = []
            for user_dict in json_resp['includes']['users']:
                uname = user_dict['username']
                userid = user_dict['id']
                location = user_dict['location'] if 'location' in user_dict else 'NA'
                session.write_transaction(self.create_user, uname,userid, location)

            for json_data in json_resp['data']:
                text = json_data['text']
                tweet_id = json_data['id']
                type = 'retweet' if text.startswith('RT @') else 'tweet'
                retweet_count  = json_data['public_metrics'] ['retweet_count']
                if type=='tweet' :
                    session.write_transaction(self.create_tweet, tweet_id, text, retweet_count)

            for json_data in json_resp['data']:

                user_id = json_data['author_id']
                entities = json_data['entities']
                if 'mentions' not in entities:
                    continue
                mentions = entities['mentions']
                text = json_data['text']
                type = 'retweet' if text.startswith('RT @') else 'tweet'


                if  type=='retweet':
                    referenced_tweets = json_data['referenced_tweets']
                    referenced_tweet = [referenced_tweet for referenced_tweet in referenced_tweets if
                                        referenced_tweet['type'] == 'retweeted']
                    for mention in mentions:
                        if   mention['start']==3  :
                            relation_type = 'retweets'
                            referenced_tweet_id = referenced_tweet[0]['id']
                            all_referenced_tweet_ids.append(referenced_tweet_id)
                            session.write_transaction(self.create_user_retweet_relation, user_id, referenced_tweet_id, relation_type)
                else :
                    session.write_transaction(self.create_user_retweet_relation, user_id, json_data['id'],
                                              'tweets')
                    for mention in mentions:
                        relation_type = 'mentioned_in'
                        session.write_transaction(self.create_user_mention_relation,  mention['id'], mention['username'], json_data['id'], relation_type)
            start =0
            end = 100

            collector = TweetCollector()
            # create a relation for all the referenced tweets

            while start<len(all_referenced_tweet_ids) :
                ref_ids = all_referenced_tweet_ids[start:end]
                ref_tweet_details =collector.collect_tweets_details(ref_ids)
                start = start+100
                end= end+100
                for user_dict in ref_tweet_details['includes']['users']:
                    uname = user_dict['username']
                    userid = user_dict['id']
                    location = user_dict['location'] if 'location' in user_dict else 'NA'
                    session.write_transaction(self.create_user, uname, userid, location)

                for tweet in ref_tweet_details['data']:
                    tweet_text = tweet['text']
                    tweet_id = tweet['id']
                    retweet_count = tweet['public_metrics']['retweet_count']
                    user_id = tweet['author_id']
                    session.write_transaction(self.create_tweet,tweet_id,tweet_text, retweet_count)
                    session.write_transaction(self.create_user_retweet_relation, user_id, tweet_id,
                                              "tweeted")

                    entities = tweet['entities']
                    if 'mentions' not in entities:
                        continue
                    mentions = entities['mentions']
                    for mention in mentions:
                        session.write_transaction(self.create_user_mention_relation,  mention['id'], mention['username'], tweet_id, "mentioned_in")


import yaml
import os

TWNEO_CONFIG = []
config_path = os.environ['twneo_config_path']
if not os.path.exists(config_path):
    print("config path does not exist using default config_yaml")
    config_path = "config.yaml"

with open (config_path ) as f :
    TWNEO_CONFIG = yaml.load(f)




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="twneo",
    version="0.0.5",
    author="Pankaj Kumar",
    author_email="kumarp@mail.smu.edu",
    description=" uses twitter api to create neo4j graphs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pankajti/twneo",
    project_urls={
        "Bug Tracker": "https://github.com/pankajti/twneo/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages( ),
    python_requires=">=3.6",
    include_package_data = True
)